import requests
import pandas as pd
import datetime as dt


appid = ""
with open("tests/openweather_app_id.txt") as f:
    appid = f.readlines()[0]

w_dict = {
    "start": "2022-09-16 12:00:00",
    "end": "2022-09-17 18:00:00",
    "interval": 6,
    "api_key": appid,
    "country": "GB",
    "postcode": "WC1",
    "tz": "GB"
}

def test_weather_obj(weather_test_dict):
    hist = WeatherHistory(start=w_dict["start"], end=w_dict["end"], interval=w_dict["interval"],
                          api_key=w_dict["api_key"], country=w_dict["country"], postcode=w_dict["postcode"],
                          tz=w_dict["tz"])
    hist.compute_weather_history()
    return hist

#TODO: Make this take the start and end times of a Survey object.
#TODO: Implement post codes instead of coordinates
class WeatherHistory:
    def __init__(self, start=None, end=None, interval=6, api_key="", country="GB", postcode="WC1", tz="",
                 units="metric"):
        # self._history_df = pd.DataFrame()
        if type(start) == str:
            self._start = dt.datetime.strptime(start, "%Y-%m-%d %H:%M:%S")
        else:
            self._start = start
        if type(end) == str:
            self._end = dt.datetime.strptime(end, "%Y-%m-%d %H:%M:%S")
        else:
            self._end = end
        self._interval = interval
        self._api_key = str(api_key)
        self._lat, self._lon = self.get_latlon(api_key=api_key, country=country, postcode=postcode)
        self._hist = None
        self._units = units

    def get_latlon(self, api_key="", country="GB", postcode=""):
        query = str("http://api.openweathermap.org/geo/1.0/zip?zip=" + postcode + "," + country + "&appid=" + api_key)
        resp = requests.get(query)
        return resp.json()["lat"], resp.json()["lon"]

    def _construct_api_call(self, timestamp):
        base = "https://api.openweathermap.org/data/3.0/onecall/timemachine?"
        query = str(base + "lat=" + str(self._lat) + "&" + "lon=" + str(self._lon) + "&" + "units=" + self._units + \
                    "&" + "dt=" + str(timestamp) + "&" + "appid=" + self._api_key)
        print(query)
        return query

    def _construct_timestamps(self):
        next_time = (self._start + dt.timedelta(hours=self._interval))
        timestamps = [int(self._start.timestamp())]
        while next_time < self._end:
            timestamps.append(int(next_time.timestamp()))
            next_time += dt.timedelta(hours=self._interval)
        return timestamps

    def _make_and_parse_api_call(self, query):
        response = requests.get(query)
        print(response.json())
        # This drops some unwanted cols like lat, lon, timezone and tz offset.
        resp_dict = response.json()["data"][0]
        del resp_dict["weather"]    # delete weather key as not useful.
        # TODO: parse 'weather' nested dict.
        return resp_dict

    def compute_weather_history(self):
        # construct timestamps
        timestamps = self._construct_timestamps()
        # make calls to API
        responses = []
        for ts in timestamps:
            print(f"ts: {ts}")
            query = self._construct_api_call(timestamp=ts)
            response_dict = self._make_and_parse_api_call(query=query)
            responses.append(pd.Series(response_dict))
        df = pd.concat(responses, axis=1).transpose()
        for col in ["dt", "sunrise", "sunset"]:
            df[col] = df[col].apply(lambda x: dt.datetime.fromtimestamp(int(x)))  # convert timestamp into datetime
        print(df)
        self._hist = df
        return df

    def get_weather_history(self):
        return self._hist



import tkinter as tk
from tkinter import filedialog ,Toplevel, IntVar, Checkbutton, Button
from tkinter import ttk
from tkinter import messagebox
from log import Log
from survey import Survey
import os
import pandas as pd
from tkinter import StringVar
#import matplotlib
#from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

#from src import log  # Assuming log is an instance of a class with the required methods

class Application(tk.Tk):
    
    def __init__(self):
        super().__init__()
        self.title("src Log Viewer")
        self.geometry("1200x800")
        import tkinter as tk

        # Create an instance of Survey
        self.survey = Survey()

        self.create_widgets()

    def create_widgets(self):

        self.grid_columnconfigure((0,1,2), weight=1)

        self.grid_columnconfigure((3,4,5), weight=2)

        self.log_label = ttk.Label(self, text="csv file name:")
        self.log_label.grid(row=0, column=0, padx=5, pady=5, sticky="e")

        self.log_file = ttk.Label(self, text="Click Browse to select")
        self.log_file.grid(row=0, column=1, padx=5, pady=5, sticky="w")

        self.browse_button1 = ttk.Button(self, text="Browse", command=self.browse_log)
        self.browse_button1.grid(row=0, column=2, padx=5, pady=5, sticky="w")

        self.analysis_label = ttk.Label(self, text="Select Analysis Type:")
        self.analysis_label.grid(row=3, column=0, padx=5, pady=10, sticky="e")

        self.analysis_var = StringVar()
        self.analysis_combobox = ttk.Combobox(self, textvariable=self.analysis_var, values=["resi_summary", "modal_l90", "lmax_spectra", "Typical_leq_spectra"])
        self.analysis_combobox.set("resi_summary")
        self.analysis_combobox.grid(row=3, column=1, padx=5, pady=10, sticky="w")
        self.analysis_var.trace("w", self.on_analysischange)

        self.parameters_label = ttk.Label(self, text="Parameters")
        self.parameters_label.grid(row=4, column=0, padx=5, pady=5, sticky="e")

        self.parameters_entry = ttk.Entry(self, width=20)
        self.parameters_entry.insert(0, 'None,None,10,2min')
        self.parameters_entry.grid(row=4, column=1, padx=5, pady=10, sticky="w")

        self.parameters_label = ttk.Label(self, text="(leq_cols,max_cols,lmax_n,lmax_t )")
        self.parameters_label.grid(row=4, column=2, padx=5, pady=5, sticky="w")

        self.execute_button = ttk.Button(self, text="Select Columns", command=self.Column_Selection_Modal)
        self.execute_button.grid(row=5, column=0, padx=5, pady=10, sticky="e")

        self.execute_button = ttk.Button(self, text="Execute", command=self.execute_code)
        self.execute_button.grid(row=5, column=2, padx=5, pady=10, sticky="w")

        self.tree = ttk.Treeview(self, show="headings")
        self.tree_scroll_y = ttk.Scrollbar(self, orient="vertical", command=self.tree.yview)
        self.tree_scroll_y.grid(row=6, column=6, sticky="ns")

        self.tree_scroll_x = ttk.Scrollbar(self, orient="horizontal", command=self.tree.xview)
        self.tree_scroll_x.grid(row=7, column=0, columnspan=3, sticky="ew")

        self.tree.configure(yscrollcommand=self.tree_scroll_y.set, xscrollcommand=self.tree_scroll_x.set)

        self.tree.grid(row=6, column=0, columnspan=4, padx=20, pady=20, sticky="nsew")
    """
      # Button to open the modal dialog
        open_dialog_button = ttk.Button(self, text="Show Time Series", command=self.open_modal_dialog)
        open_dialog_button.grid(row=5, column=2, padx=5, pady=10, sticky="w")


    # Commented out section that displays a graph
    def open_modal_dialog(self):
        dialog = Toplevel(self)
        dialog.title("Modal Dialog")

        # Make the dialog modal
        dialog.transient(self)
        dialog.grab_set()

        # Center the dialog on the screen
        window_width = self.winfo_width()
        window_height = self.winfo_height()
        window_x = self.winfo_x()
        window_y = self.winfo_y()

        dialog_width = 900
        dialog_height = 600

        position_right = int(window_x + (window_width / 2) - (dialog_width / 2))
        position_down = int(window_y + (window_height / 2) - (dialog_height / 2))

        dialog.geometry(f"{dialog_width}x{dialog_height}+{position_right}+{position_down}")
        import matplotlib.pyplot as plt

        # Create a figure and axis
        fig, ax = plt.subplots()

        # Plot the data
        #print ("----",self.df.columns[1][0],self.df.columns[1][1] )        
        #print ("----",type(self.df.columns[1]))
        
        leq_a_column = [col for col in self.df.columns if isinstance(col, tuple) and col[0] == "Leq" and col[1] == "A"]
        if not leq_a_column:
            messagebox.showerror("Error", "No columns found with ('Leq', 'A') in the header.")
            return

        l90_a_columns = [col for col in self.df.columns if isinstance(col, tuple) and col[0] == "L90" and col[1] == "A"]
        if not l90_a_columns:
            messagebox.showerror("Error", "No columns found with ('L90', 'A') in the header.")
            return


        self.df["Date"] = pd.to_datetime(self.df.index)
        #print(self.df)

        ax.plot(self.df["Date"], self.df[leq_a_column[0]], label='LAeq')
        ax.plot(self.df["Date"], self.df[l90_a_column[0]], label='LA90')
        #ax.plot(self.df['Index'], self.df['L90 A'], label='la90')

        # Format the x-axis to show dates properly
        fig.autofmt_xdate()

        # Add labels and title
        ax.set_xlabel('Date')
        ax.set_ylabel('Values')
        ax.set_title('Time Series Line Chart')
        ax.legend()

        # Create a canvas to display the plot in the Tkinter dialog
        canvas = FigureCanvasTkAgg(fig, master=dialog)
        canvas.draw()
        canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)
        #else:
        #    messagebox.showerror("Error", "DataFrame does not contain required columns: 'date', 'Laeq', 'la90'")

        # OK button to close the dialog
        ok_button = ttk.Button(dialog, text="OK", command=dialog.destroy)
        ok_button.pack(pady=20)

    """

    def on_analysischange(self, *args):
        analysis_type = self.analysis_combobox.get()
        if analysis_type == "resi_summary":
            self.parameters_entry.config(state='normal')
        else:
            self.parameters_entry.config(state='disabled')
        return
 
  
    def browse_log(self):
        self.logpath = os.getcwd  ()
        file_path = tk.filedialog.askopenfilename(initialdir=self.logpath, filetypes=[("CSV files", "*.csv")])
        if file_path:
            self.logname = os.path.basename(file_path)
            self.logpath = os.path.dirname(file_path)
            self.log_file.config(text=self.logname)

            strPath = self.logpath + "\\" + self.logname
            self.log  = Log(path=strPath)
            self.survey.add_log(data=self.log, name="Position 1")

            self.df = self.log.get_data()

            # Clear the treeview
            for item in self.tree.get_children():
                self.tree.delete(item)

            # Round the dataframe values to 1 decimal place, except for the column headers
            self.df = self.df.round(1)
 
            # Insert new data into the treeview
            self.tree["columns"] = ["Index"] + list(self.df.columns)
            self.tree.heading("Index", text="Index")
            self.tree.column("Index", width=150, stretch=True)

            i = 0 
            for col in self.df.columns:
                self.tree.heading(col, text=col)
                i=i+1
                if i < 12:
                    self.tree.column(col, width=75,stretch=True, anchor="center")
                else:
                    self.tree.column(col, width=0,stretch=False, anchor="center")
          
            for index, row in self.df.iterrows():
                self.tree.insert("", "end", values=[index] + list(row))
            return
 
    def execute_code(self):
        analysis_type = self.analysis_combobox.get()
        parameters = self.parameters_entry.get()

        try:
            df = pd.DataFrame()
            if analysis_type == "resi_summary":
                params = parameters.split(",")
                p = [None if x == "None" else x for x in params]
                df =self.survey.resi_summary(leq_cols=p[0], max_cols=p[1], lmax_n=int(params[2]), lmax_t=params[3])  
                print ("df id a ",type(df))
            elif analysis_type == "modal_l90":
                df = self.survey.modal()
            elif analysis_type == "lmax_spectra":
                df = self.survey.lmax_spectra()
            elif analysis_type == "Leq_spectra":
                df = self.survey.leq_spectra()
            else:
                messagebox.showerror("Error", "Please select an analysis type.")
                return
        except Exception as e:
            messagebox.showerror("Error", f"An error occurred: {e}")
            return
        
        # Clear the treeview
        for item in self.tree.get_children():
            self.tree.delete(item)

        # Set new column headers based on the dataframe
        #self.tree["columns"] = list(df.columns)

        # Insert new data into the treeview
        self.tree["columns"] = ["Index"] + list(df.columns)
        self.tree.heading("Index", text="Index")
        self.tree.column("Index", width=150, anchor="center", stretch=True)
        

        for col in df.columns:
            self.tree.heading(col, text=col)
            self.tree.column(col, width=75, anchor="center", stretch=True)
   
        # Insert new data into the treeview
        for index, row in df.iterrows():
            self.tree.insert("", "end", values=[index] + list(row))

        # Copy the DataFrame to the clipboard
        df.to_clipboard(index=True)
        messagebox.showinfo("Success", "DataFrame copied to clipboard.")


    def Column_Selection_Modal(self):   
        # Create a modal dialog
        #print ("def Column_Selection_Modal(self):")

        dialog = Toplevel(self)
        dialog.title("Select Columns")

        # Make the dialog modal
        dialog.transient(self)
        dialog.grab_set()

        #self.window.update_idletasks()
        window_width = self.winfo_width()
        window_height = self.winfo_height()
        window_x = self.winfo_x()
        window_y = self.winfo_y()

        dialog_width = 300
        dialog_height = 600

        position_right = int(window_x + (window_width / 2) - (dialog_width / 2))
        position_down = int(window_y + (window_height / 2) - (dialog_height / 2))

        dialog.geometry(f"{dialog_width}x{dialog_height}+{position_right}+{position_down}")

        # Configure grid rows and columns
        for i in range(12):  # Adjust the range as needed
            dialog.rowconfigure(i,weight =1,uniform = 'a')

        for i in range(3):  # Adjust the range as needed
            dialog.grid_columnconfigure(i,  weight =1,minsize = 20,uniform = 'a')

        # Get the column identifiers
        column_ids = self.tree["columns"]
        #print ("col ids",column_ids)

        # Get the column headers
        column_headers = [self.tree.heading(col)["text"] for col in column_ids]

        # Get the column widths
        column_widths = [self.tree.column(col)["width"] for col in column_ids]


        self.column_vars = {}
        for i, column in enumerate(column_headers):
            # Check if the column is visible in self.tree
            self.column_vars[column] = IntVar(master=dialog, value=1 if column_widths[i] > 0 else 0)
        self.cb = []

        # Create checkboxes for each column
        for i, column in enumerate(column_headers):
            #print(f"Column: {column}, Value: {self.column_vars[column].get()}")            
            self.cb.append( Checkbutton(dialog, text=column, variable=self.column_vars[column]))
            #self.cb[i].grid(row=i+1, column=(i-1)/10, sticky='w')
            self.cb[i].grid(row=i%10, column=i//10, sticky='w')

        for i, column in enumerate(column_headers):
            self.column_vars[column].set(1 if column_widths[i] > 0 else 0)

        # OK button to apply the selection
        ttk.Button(dialog, text="OK", command=lambda: self.apply_column_selection(dialog)).grid(row=13, column=1, ipady=10)
       # Cancel button
        def on_cancel():
            #print("Cancel clicked")
            dialog.destroy()

        cancel_button = ttk.Button(dialog, text="Cancel", command=on_cancel)
        cancel_button.grid(row=13,column = 2, sticky = "w",ipady = 10)   

        return     

    def apply_column_selection(self, dialog):

        #print("def apply_column_selection")

        # Get the selected columns
        selected_columns = [column for column, var in self.column_vars.items() if var.get() == 1]

        # Set the column widths based on the selection
        for column in self.tree["columns"]:
            column_text = self.tree.heading(column)["text"]
            if column_text in selected_columns:
                self.tree.column(column, width=250,stretch=True, anchor="center")  # Set to a default width
            else:
                self.tree.column(column, width=0,stretch=False, anchor="center")  # Hide the column

        # Display the selected columns in the listbox
        #for item in self.tree:
        #    display_text = " | ".join(str(item[column]) for column in selected_columns)
        #    self.listbox.insert(END, display_text)
        # Close the dialog
        dialog.destroy()

if __name__ == "__main__":
    app = Application()
    app.mainloop()


import pandas as pd
import numpy as np
from .weather import WeatherHistory


class Survey:
    """
    Survey Class is an overarching class which takes multiple Log objects and processes and summarises them together.
    This should be the main interface for user interaction with their survey data.
    """

    # ###########################---PRIVATE---######################################

    def __init__(self):
        self._logs = {}

    def _insert_multiindex(self, df=None, super=None, name1="Position", name2="Date"):
        subs = df.index.to_list()   # List of subheaders (dates)
        # Super should be the position name (key from master dictionary)
        tuples = [(super, sub) for sub in subs]
        idx = pd.MultiIndex.from_tuples(tuples, names=[name1, name2])
        return df.set_index(idx, inplace=False)

    def _insert_header(self, df=None, new_head_list=None, header_idx=None):
        cols = df.columns.to_list()
        new_cols = [list(c) for c in zip(*cols)]
        new_cols.insert(header_idx, new_head_list)
        df.columns = new_cols
        return df

    # ###########################---PUBLIC---######################################

    def set_periods(self, times=None):
        """
        Set the daytime, evening and night-time periods of all Log objects in the Survey.
        To disable evening periods, simply set it the same as night-time.
        :param times: A dictionary with strings as keys and integer tuples as values.
        The first value in the tuple represents the hour of the day that period starts at (24hr clock), and the
        second value represents the minutes past the hour.
        e.g. for daytime from 07:00 to 19:00, evening 19:00 to 23:00 and night-time 23:00 to 07:00,
        times = {"day": (7, 0), "evening": (19, 0), "night": (23, 0)}
        NOTES:
        Night-time must cross over midnight. (TBC experimentally).
        Evening must be between daytime and night-time. To
        :return: None.
        """
        if times is None:
            times = {"day": (7, 0), "evening": (23, 0), "night": (23, 0)}
        for key in self._logs.keys():
            self._logs[key].set_periods(times=times)

    def add_log(self, data=None, name=""):
        """
        Add a Log object to the Survey object.
        :param data: Initialised Log object
        :param name: Name of the position, e.g. "A1"
        :return: None.
        """
        self._logs[name] = data

    def get_periods(self):
        """
        Check the currently-set daytime, evening and night-time periods for each Log object in the Survey.
        :return: Tuples of start times.
        """
        periods = {}
        for key in self._logs.keys():
            periods[key] = self._logs[key].get_period_times()
        return periods

    def resi_summary(self, leq_cols=None, max_cols=None, lmax_n=10, lmax_t="2min"):
        """
        Get a dataframe summarising the parameters relevant to assessment of internal ambient noise levels in
        UK residential property assessments. Daytime and night-time Leqs, and nth-highest Lmax values all presented
        in a succinct table. These will be summarised as per the daytime, evening and night-time periods set (default
        daytime 07:00 to 23:00 and night-time 23:00 to 07:00).
        The date of the Lmax values are presented for the night-time period beginning on that date. i.e. an Lmax
        on 20/12/2024 would have occurred in the night-time period starting on that date and ending the following
        morning.
        :param leq_cols: List of tuples. The columns on which to perform Leq calculations. This can include L90
        columns, or spectral values. e.g.  leq_cols = [("Leq", "A"), ("L90", "125")]
        :param max_cols: List of tuples. The columns on which to get the nth-highest values.
        Default max_cols = [("Lmax", "A")]
        :param lmax_n: Int. The nth-highest value for max_cols. Default 10 for 10th-highest.
        :param lmax_t: String. This is the time period over which to compute nth-highest Lmax values.
        e.g. "2min" computes the nth-highest Lmaxes over 2-minute periods. Note that the chosen period must be
        equal to or more than the measurement period. So you cannot measure in 5-minute periods and request 2-minute
        Lmaxes.
        :return: A dataframe presenting a summary of the Leq and Lmax values requested.
        """
        combi = pd.DataFrame()
        if leq_cols is None:
            leq_cols = [("Leq", "A")]
        if max_cols is None:
            max_cols = [("Lmax", "A")]
        for key in self._logs.keys():
            log = self._logs[key]
            combined_list = []
            # Day
            days = log._leq_by_date(log._get_period(data=log.get_antilogs(), period="days"), cols=leq_cols)
            days.sort_index(inplace=True)
            combined_list.append(days)
            period_headers = ["Daytime" for i in range(len(leq_cols))]
            # Evening
            if log.is_evening():
                evenings = log._leq_by_date(log._get_period(data=log.get_antilogs(), period="evenings"), cols=leq_cols)
                evenings.sort_index(inplace=True)
                combined_list.append(evenings)
                for i in range(len(leq_cols)):
                    period_headers.append("Evening")
            # Night Leq
            nights = log._leq_by_date(log._get_period(data=log.get_antilogs(), period="nights"), cols=leq_cols)
            nights.sort_index(inplace=True)
            combined_list.append(nights)
            for i in range(len(leq_cols)):
                period_headers.append("Night-time")
            # Night max
            maxes = log.as_interval(t=lmax_t)
            maxes = log._get_period(data=maxes, period="nights", night_idx=True)
            maxes = log.get_nth_high_low(n=lmax_n, data=maxes)[max_cols]
            maxes.sort_index(inplace=True)
            #  +++
            # SS Feb2025  - Code changed to prevent exception
            #maxes.index = maxes.index.date
            try:
                maxes.index = pd.to_datetime(maxes.index)
                maxes.index = maxes.index.date
            except Exception as e:
                print(f"Error converting index to date: {e}")
            # SSS ---
            maxes.index.name = None
            combined_list.append(maxes)
            for i in range(len(max_cols)):
                period_headers.append("Night-time")
            summary = pd.concat(objs=combined_list, axis=1)
            summary = self._insert_multiindex(df=summary, super=key)
            combi = pd.concat(objs=[combi, summary], axis=0)
        combi = self._insert_header(df=combi, new_head_list=period_headers, header_idx=0)
        return combi

    def modal(self, cols=None, by_date=False, day_t="60min", evening_t="60min", night_t="15min"):
        """
        Get a dataframe summarising Modal L90 values for each time period, as suggested by BS 4142:2014.
        Currently, this method rounds the values to 0 decimal places by default and there is no alternative
        implementation.
        Note that this function will estimate L90s as a longer value by performing an Leq computation on them.
        The measured data in Logs must be smaller than or equal to the desired period, i.e. you can't measure in 15-
        minute periods and request 5-minute modal values.
        :param cols: List of tuples of the columns desired. This does not have to be L90s, but can be any column.
        :param by_date: Bool. If True, group the modal values by date. If False, present one modal value for each
        period.
        :param day_t:  String. Measurement period T. i.e. daytime measurements will compute modal values of
        L90,60min by default.
        :param evening_t: String. Measurement period T. i.e. evening measurements will compute modal values of
        L90,60min by default, unless evenings are disabled (which they are by default).
        :param night_t: Measurement period T. i.e. night-time measurements will compute modal values of
        L90,15min by default.
        :return: A dataframe of modal values for each time period.
        """
        if cols is None:
            cols = [("L90", "A")]
        combi = pd.DataFrame()
        period_headers = []
        for key in self._logs.keys():
            # Key is the name of the measurement position
            log = self._logs[key]
            pos_summary = []
            # Daytime
            period_headers = ["Daytime"]
            days = log.get_modal(data=log._get_period(data=log.as_interval(t=day_t), period="days"), by_date=by_date, cols=cols)
            days.sort_index(inplace=True)
            pos_summary.append(days)
            # Evening
            if log.is_evening():
                period_headers.append("Evening")
                evenings = log.get_modal(data=log._get_period(data=log.as_interval(t=evening_t), period="evenings"), by_date=by_date, cols=cols)
                evenings.sort_index(inplace=True)
                pos_summary.append(evenings)
            # Night time
            nights = log.get_modal(data=log._get_period(data=log.as_interval(t=night_t), period="nights"), by_date=by_date, cols=cols)
            nights.sort_index(inplace=True)
            pos_summary.append(nights)
            period_headers.append("Night-time")
            pos_df = pd.concat(pos_summary, axis=1)
            pos_df = self._insert_multiindex(pos_df, super=key)
            combi = pd.concat([combi, pos_df], axis=0)
        combi = self._insert_header(df=combi, new_head_list=period_headers, header_idx=0)
        return combi

    def counts(self, cols=None, day_t="60min", evening_t="60min", night_t="15min"):
        if cols is None:
            cols = [("L90", "A")]
        combi = pd.DataFrame()
        period_headers = []
        for key in self._logs.keys():
            # Key is the name of the measurement position
            log = self._logs[key]
            pos_summary = []
            # Daytime
            period_headers = ["Daytime"]
            days = log.counts(data=log._get_period(data=log.as_interval(t=day_t), period="days"), cols=cols)
            days.sort_index(inplace=True)
            pos_summary.append(days)
            # Evening
            if log.is_evening():
                period_headers.append("Evening")
                evenings = log.counts(data=log._get_period(data=log.as_interval(t=evening_t), period="evenings"), cols=cols)
                evenings.sort_index(inplace=True)
                pos_summary.append(evenings)
            # Night time
            nights = log.counts(data=log._get_period(data=log.as_interval(t=night_t), period="nights"), cols=cols)
            nights.sort_index(inplace=True)
            pos_summary.append(nights)
            period_headers.append("Night-time")
            pos_df = pd.concat(pos_summary, axis=1)
            pos_df = self._insert_multiindex(pos_df, super=key)
            combi = pd.concat([combi, pos_df], axis=0)
        combi = self._insert_header(df=combi, new_head_list=period_headers, header_idx=0)
        return combi

    def lmax_spectra(self, n=10, t="2min", period="nights"):
        """
        Get spectral data for the nth-highest Lmax values during a given time period.
        This computes Lmax Event spectra. Lmax Hold spectra has not yet been implemented.
        Assumptions and inputs as per Survey.resi_summary() method.
        IMPORTANT: The dates of the Lmax values are presented for the night-time period beginning on that date.
        This means that for early morning timings, the date is behind by one day.
        e.g. an Lmax presented as occurring at 20/12/2024 at 01:22 would have occurred at 21/12/2024 at 01:22.
        :param n: Int. Nth-highest Lmax. Default 10th-highest.
        :param t: String. This is the time period over which to compute nth-highest Lmax values.
        e.g. "2min" computes the nth-highest Lmaxes over 2-minute periods. Note that the chosen period must be
        equal to or more than the measurement period. So you cannot measure in 5-minute periods and request 2-minute
        Lmaxes.
        :param period: String. "days", "evenings" or "nights"
        :return: Dataframe of nth-highest Lmax Event spectra.
        """
        combi = pd.DataFrame()
        # TODO: The night-time timestamp on this is sometimes out by a minute.
        for key in self._logs.keys():
            log = self._logs[key]
            combined_list = []
            maxes = log.get_nth_high_low(n=n, data=log._get_period(data=log.as_interval(t=t), period=period))[["Lmax", "Time"]]
            maxes.sort_index(inplace=True)
            combined_list.append(maxes)
            summary = pd.concat(objs=combined_list, axis=1)
            summary = self._insert_multiindex(df=summary, super=key)
            combi = pd.concat(objs=[combi, summary], axis=0)
        return combi

    # TODO: get_lowest_l90

    def leq_spectra(self, leq_cols=None):
        """
        Compute Leqs over daytime, evening and night-time periods.
        This is an overall Leq, and does not group Leqs by date.
        :param leq_cols: List of strings or List of Tuples.
        For all Leq columns, use ["Leq"]. For specific columns, use list of tuples [("Leq", "A"), ("Leq", 125)]
        :return: A dataframe with a continuous Leq computation across dates, for each time period.
        """
        #TODO: C:\Users\tonyr\PycharmProjects\src\tests.py:674: FutureWarning: The behavior of pd.concat with len(keys) != len(objs) is deprecated. In a future version this will raise instead of truncating to the smaller of the two sequences combi = pd.concat(all_pos, axis=1, keys=["UA1", "UA2"])
        all_pos = []
        if leq_cols is None:
            leq_cols = ["Leq"]
        for key in self._logs.keys():
            log = self._logs[key]
            # Day
            days = log._get_period(data=log.get_antilogs(), period="days")
            days = days[leq_cols].apply(lambda x: np.round(10*np.log10(np.mean(x)), DECIMALS))
            # Night-time
            nights = log._get_period(data=log.get_antilogs(), period="nights")
            nights = nights[leq_cols].apply(lambda x: np.round(10*np.log10(np.mean(x)), DECIMALS))
            df = pd.DataFrame
            # Evening
            if log.is_evening():
                evenings = log._get_period(data=log.get_antilogs(), period="evenings")
                evenings = evenings[leq_cols].apply(lambda x: np.round(10 * np.log10(np.mean(x)), DECIMALS))
                df = pd.concat([days, evenings, nights], axis=1, keys=["Daytime", "Evening", "Night-time"])
            else:
                df = pd.concat([days, nights], axis=1, keys=["Daytime", "Night-time"])
            all_pos.append(df)
        combi = pd.concat(all_pos, axis=1, keys=["UA1", "UA2"])
        combi = combi.transpose()
        return combi

    def get_start_end(self):
        starts = [self._logs[key].get_start() for key in self._logs.keys()]
        ends = [self._logs[key].get_end() for key in self._logs.keys()]
        return min(starts), max(ends)

    # def typical_leq_spectra(self, leq_cols=None):
    #     """
    #     DEPRECATED 2025/06/05. Replaced by .leq_spectra() **TT**
    #     Compute Leqs over daytime, evening and night-time periods.
    #     This is an overall Leq, and does not group Leqs by date.
    #     :param leq_cols: List of strings or List of Tuples.
    #     For all Leq columns, use ["Leq"]. For specific columns, use list of tuples [("Leq", "A"), ("Leq", 125)]
    #     :return: A dataframe with a continuous Leq computation across dates, for each time period.
    #     """
    #     combi = pd.DataFrame()
    #     if leq_cols is None:
    #         leq_cols = ["Leq"]
    #     for key in self._logs.keys():
    #         log = self._logs[key]
    #         combined_list = []
    #         # Day
    #         days = log._get_period(data=log.get_antilogs(), period="days")
    #         days = days[leq_cols].apply(lambda x: np.round(10*np.log10(np.mean(x)), DECIMALS))
    #         #days.sort_index(inplace=True)
    #         combined_list.append(days)
    #         period_headers = ["Daytime" for i in range(len(leq_cols))]
    #         # Evening
    #         if log.is_evening():
    #             evenings = log._get_period(data=log.get_antilogs(), period="evenings")
    #             evenings = evenings[leq_cols].apply(lambda x: np.round(10*np.log10(np.mean(x)), DECIMALS))
    #             evenings.sort_index(inplace=True)
    #             combined_list.append(evenings)
    #             for i in range(len(leq_cols)):
    #                 period_headers.append("Evening")
    #         # Night Leq
    #         nights = log._get_period(data=log.get_antilogs(), period="nights")
    #         nights = nights[leq_cols].apply(lambda x: np.round(10*np.log10(np.mean(x)), DECIMALS))
    #         nights.sort_index(inplace=True)
    #         combined_list.append(nights)
    #         for i in range(len(leq_cols)):
    #             period_headers.append("Night-time")
    #         summary = pd.concat(objs=combined_list, axis=1)
    #         summary = self._insert_multiindex(df=summary, super=key)
    #         combi = pd.concat(objs=[combi, summary], axis=0)
    #     new_head_dict = {}
    #     for i in range(len(period_headers)):
    #         new_head_dict[i] = period_headers[i]
    #     combi.rename(columns=new_head_dict, inplace=True)
    #     #combi = combi.transpose()
    #     return combi
